import { ReactNode } from 'react';

import { TabInitialState, useTabState } from 'reakit/Tab';

import { TabsButton } from './button';
import { TabsContext } from './context';
import { TabsList } from './list';
import { TabsPanel } from './panel';

interface TabsProps {
  children?: ReactNode;
  className?: string;
  options?: TabInitialState;
}

export function Tabs({ children, className, options = {} }: TabsProps) {
  const state = useTabState(options);
  return (
    <div className={className}>
      <TabsContext.Provider value={state}>{children}</TabsContext.Provider>
    </div>
  );
}

Tabs.defaultProps = {
  children: null,
  className: null,
  options: {},
};

Tabs.Button = TabsButton;
Tabs.List = TabsList;
Tabs.Panel = TabsPanel;
